@@ -1,10 +1,10 @@
 ;;; sql.el --- specialized comint.el for SQL interpreters
 
-;; Copyright (C) 1998, 1999  Free Software Foundation, Inc.
+;; Copyright (C) 1998, 1999, 2000  Free Software Foundation, Inc.
 
 ;; Author: Alex Schroeder <alex@gnu.org>
 ;; Maintainer: Alex Schroeder <alex@gnu.org>
-;; Version: 1.4.13
+;; Version: 1.4.21
 ;; Keywords: comm languages processes
 
 ;; This file is part of GNU Emacs.
@@ -58,10 +58,8 @@
 ;; modes derived from comint mode.  This makes these modes easier to
 ;; use.
 
-;; sql-mode can be used to enable syntactic hilighting for SQL
-;; statements in another buffer.  SQL statements can then be sent to
-;; the SQL process in the SQLi buffer.  sql-mode has already been
-;; used as a template to a simple PL/SQL mode.
+;; sql-mode can be used to keep editing SQL statements.  The SQL
+;; statements can be sent to the SQL process in the SQLi buffer.
 
 ;; For documentation on the functionality provided by comint mode, and
 ;; the hooks available for customising it, see the file `comint.el'.
@@ -85,6 +83,8 @@
 ;; only appear after the process is exited.  This makes things
 ;; somewhat unreliable.
 
+;; ChangeLog available on request.
+
 ;;; To Do:
 
 ;; Add better hilight support for other brands; there is a bias towards
@@ -148,6 +148,20 @@ Customizing your password will store it in your ~/.emacs file."
 
 ;; misc customization of sql.el behaviour
 
+(defcustom sql-electric-stuff nil
+  "Treat some input as electric.
+If set to the symbol `semicolon', then hitting `;' will send current
+input in the SQLi buffer to the process.
+If set to the symbol `go', then hitting `go' on a line by itself will
+send current input in the SQLi buffer to the process.
+If set to nil, then you must use \\[comint-send-input] in order to send
+current input in the SQLi buffer to the process."
+  :type '(choice (const :tag "Nothing" nil)
+		 (const :tag "The semikolon `;'" semicolon)
+		 (const :tag "The string `go' by itself" go))
+  :version "20.8"
+  :group 'SQL)
+
 (defcustom sql-pop-to-buffer-after-send-region nil
   "*If t, pop to the buffer SQL statements are sent to.
 
@@ -182,8 +196,10 @@ exiting the SQL interpreter in an SQLi buffer will write the input
 history to the specified file.  Starting a new process in a SQLi buffer
 will read the input history from the specified file.
 
-This is used to locally set `comint-input-ring-file-name' when reading
-or writing the input history."
+This is used to initialize `comint-input-ring-file-name'.
+
+Note that the size of the input history is determined by the variable
+`comint-input-ring-size'."
   :type '(choice (const :tag "none" nil)
 		 (file))
   :group 'SQL)
@@ -195,15 +211,12 @@ If set to \"\\n\", each line in the history file will be interpreted as
 one command.  Multi-line commands are split into several commands when
 the input ring is initialized from a history file.
 
-This variable used to locally set `comint-input-ring-separator' when
-reading or writing the history file.  `comint-input-ring-separator' is
-not yet part of Emacs; if your Emacs does not have it, setting
-`sql-input-ring-separator' will have no effect.  In that case multiline
-commands will be split into several commands when the input history is
-read, as if you had set `sql-input-ring-separator' to \"\\n\".
-
-The source code contains a link to a homepage that might have a patch
-for comint.el to download."
+This variable used to initialize `comint-input-ring-separator'.
+`comint-input-ring-separator' is part of Emacs 21; if your Emacs
+does not have it, setting `sql-input-ring-separator' will have no
+effect.  In that case multiline commands will be split into several
+commands when the input history is read, as if you had set
+`sql-input-ring-separator' to \"\\n\"."
   :type 'string
   :group 'SQL)
 
@@ -245,6 +258,7 @@ The program can also specify a TCP connection.  See `make-comint'."
 (defcustom sql-oracle-options nil
   "*List of additional options for `sql-oracle-program'."
   :type '(repeat string)
+  :version "20.8"
   :group 'SQL)
 
 ;; Customisation for MySql
@@ -324,6 +338,16 @@ The program can also specify a TCP connection.  See `make-comint'."
   :type 'file
   :group 'SQL)
 
+(defcustom sql-postgres-options '("--pset" "pager=off")
+  "*List of additional options for `sql-postgres-program'.
+The default setting includes the --pset option which breaks
+older versions of the psql client (such as version 6.5.3).
+If you want the psql to prompt you for a user name, add the
+string \"-u\" to the list of options."
+  :type '(repeat string)
+  :version "20.8"
+  :group 'SQL)
+
 
 
 ;;; Variables which do not need customization
@@ -348,11 +372,6 @@ See `sql-interactive-mode' for more on multiple sessions.  If you want
 to change the SQLi buffer a SQL mode sends its SQL strings to, change
 the local value of `sql-buffer' using \\[sql-set-sqli-buffer].")
 
-(defvar sql-server-type nil
-  "Server type used in the current buffer.
-The default-value of this variable holds the server type of
-the last SQLi buffer created.")
-
 (defvar sql-prompt-regexp nil
   "Prompt used to initialize `comint-prompt-regexp'.
 
@@ -368,33 +387,31 @@ You can change it on `sql-interactive-mode-hook'.")
 
 Used by `sql-rename-buffer'.")
 
-;; Keymap for sql-interactive-mode, based on comint-mode-map.
+;; Keymap for sql-interactive-mode.
 
-(if (not (string-match "XEmacs\\|Lucid" emacs-version))
-    (defvar sql-interactive-mode-map
-      (let ((map (nconc (make-sparse-keymap) comint-mode-map)))
-	(define-key map "\C-j" 'sql-accumulate-and-indent)
-	(define-key map "\C-c\C-w" 'sql-copy-column)
-	map)
-      "Mode map used for `sql-interactive-mode'.")
-  ;; XEmacs
-  (defvar sql-interactive-mode-map nil)
-  (if (not sql-interactive-mode-map)
-      (let ((map (make-keymap)))
-	(set-keymap-parents map (list comint-mode-map))
-	(set-keymap-name map 'sql-interactive-mode-map)
-	(define-key map "\C-j" 'sql-accumulate-and-indent)
-	(define-key map "\C-c\C-w" 'sql-copy-column)
-	(setq sql-interactive-mode-map map))))
+(defvar sql-interactive-mode-map 
+  (let ((map (make-sparse-keymap)))
+    (if (functionp 'set-keymap-parent)
+	(set-keymap-parent map comint-mode-map); Emacs
+      (set-keymap-parents map (list comint-mode-map))); XEmacs
+    (if (functionp 'set-keymap-name)
+	(set-keymap-name map 'sql-interactive-mode-map)); XEmacs
+    (define-key map (kbd "C-j") 'sql-accumulate-and-indent)
+    (define-key map (kbd "C-c C-w") 'sql-copy-column)
+    (define-key map (kbd "O") 'sql-magic-go)
+    (define-key map (kbd "o") 'sql-magic-go)
+    (define-key map (kbd ";") 'sql-magic-semicolon)
+    map)
+  "Mode map used for `sql-interactive-mode'.
+Based on `comint-mode-map'.")
 
 ;; Keymap for sql-mode.
 
 (defvar sql-mode-map
   (let ((map (make-sparse-keymap)))
-    (define-key map "\C-c\C-c" 'sql-send-paragraph)
-    (define-key map "\C-c\C-r" 'sql-send-region)
-    (define-key map "\C-c\C-b" 'sql-send-buffer)
-    (define-key map "\t" 'indent-relative)
+    (define-key map (kbd "C-c C-c") 'sql-send-paragraph)
+    (define-key map (kbd "C-c C-r") 'sql-send-region)
+    (define-key map (kbd "C-c C-b") 'sql-send-buffer)
     map)
   "Mode map used for `sql-mode'.")
 
@@ -406,7 +423,9 @@ Used by `sql-rename-buffer'.")
  '("SQL"
    ["Send Paragraph" sql-send-paragraph (and (buffer-live-p sql-buffer)
 					     (get-buffer-process sql-buffer))]
-   ["Send Region" sql-send-region (and mark-active
+   ["Send Region" sql-send-region (and (or (and (boundp 'mark-active); Emacs
+						mark-active)
+					   (mark)); XEmacs
 				       (buffer-live-p sql-buffer)
 				       (get-buffer-process sql-buffer))]
    ["Send Buffer" sql-send-buffer (and (buffer-live-p sql-buffer)
@@ -518,21 +537,31 @@ to add functions and PL/SQL keywords.")
 "become" "before" "block" "body" "cache" "cancel" "cascade" "change"
 "checkpoint" "compile" "constraint" "constraints" "contents"
 "controlfile" "cycle" "database" "datafile" "dba" "disable" "dismount"
-"dump" "each" "enable" "events" "except" "exceptions" "execute"
-"explain" "extent" "externally" "flush" "force" "freelist" "freelists"
-"function" "groups" "including" "initrans" "instance" "layer" "link"
-"lists" "logfile" "manage" "manual" "maxdatafiles" "maxinistances"
-"maxlogfiles" "maxloghistory" "maxlogmembers" "maxtrans" "maxvalue"
-"minextents" "minvalue" "mount" "new" "next" "noarchivelog" "nocache"
-"nocycle" "nomaxvalue" "nominvalue" "none" "noorder" "noresetlogs"
-"normal" "nosort" "off" "old" "only" "optimal" "own" "package"
-"parallel" "pctincrease" "pctused" "plan" "private" "profile" "quota"
-"read" "recover" "referencing" "resetlogs" "restricted" "reuse" "role"
-"roles" "savepoint" "scn" "segment" "sequence" "shared" "snapshot"
-"sort" "statement_id" "statistics" "stop" "storage" "switch" "system"
+"dump" "each" "else" "elsif" "enable" "events" "except" "exceptions"
+"execute" "exit" "explain" "extent" "externally" "false" "flush" "force"
+"freelist" "freelists" "function" "groups" "if" "including" "initrans"
+"instance" "layer" "link" "lists" "logfile" "loop" "manage" "manual"
+"maxdatafiles" "maxinistances" "maxlogfiles" "maxloghistory"
+"maxlogmembers" "maxtrans" "maxvalue" "minextents" "minvalue" "mount"
+"new" "next" "noarchivelog" "nocache" "nocycle" "nomaxvalue"
+"nominvalue" "none" "noorder" "noresetlogs" "normal" "nosort" "off"
+"old" "only" "optimal" "others" "out" "own" "package" "parallel"
+"pctincrease" "pctused" "plan" "pragma" "private" "profile" "quota"
+"raise" "read" "recover" "referencing" "resetlogs" "restrict_references"
+"restricted" "return" "returning" "reuse" "rnds" "rnps" "role" "roles"
+"savepoint" "scn" "segment" "sequence" "shared" "snapshot" "sort"
+"statement_id" "statistics" "stop" "storage" "subtype" "switch" "system"
 "tables" "tablespace" "temporary" "thread" "time" "tracing"
-"transaction" "triggers" "truncate" "under" "unlimited" "until" "use"
-"using" "when" "write") t) "\\b")))
+"transaction" "triggers" "true" "truncate" "type" "under" "unlimited"
+"until" "use" "using" "when" "while" "wnds" "wnps" "write") t) "\\b")))
+	(oracle-warning-words (eval-when-compile
+				 (concat "\\b"
+					 (regexp-opt '(
+"cursor_already_open" "dup_val_on_index" "exception" "invalid_cursor"
+"invalid_number" "login_denied" "no_data_found" "not_logged_on"
+"notfound" "others" "pragma" "program_error" "storage_error"
+"timeout_on_resource" "too_many_rows" "transaction_backed_out"
+"value_error" "zero_divide") t) "\\b")))
 	(oracle-reserved-words (eval-when-compile
 				 (concat "\\b"
 					 (regexp-opt '(
@@ -549,7 +578,8 @@ to add functions and PL/SQL keywords.")
 				(regexp-opt '(
 ;; Oracle Keywords that look like types
 ;; Oracle Reserved Words that look like types
-"date" "decimal" "rowid" "varchar" "varchar2") t) "\\b")))
+"binary_integer" "blob" "boolean" "constant" "date" "decimal" "rowid"
+"varchar" "varchar2") t) "\\b")))
 	(oracle-builtin-functions (eval-when-compile
 			(concat "\\b"
 				(regexp-opt '(
@@ -564,10 +594,11 @@ to add functions and PL/SQL keywords.")
 "rtrim" "sign" "sin" "sinh" "soundex" "sqlcode" "sqlerrm" "sqrt"
 "stddev" "sum" "substr" "substrb" "tan" "tanh" "to_char"
 "to_date" "to_label" "to_multi_byte" "to_number" "to_single_byte"
-"translate" "trunc" "uid" "upper" "userenv" "variance" "vsize") t) "\\b"))))
+"translate" "trim" "trunc" "uid" "upper" "userenv" "variance" "vsize") t) "\\b"))))
     (setq sql-mode-oracle-font-lock-keywords
 	  (append sql-mode-ansi-font-lock-keywords
 		  (list (cons oracle-keywords 'font-lock-function-name-face)
+			(cons oracle-warning-words 'font-lock-warning-face)
 			(cons oracle-reserved-words 'font-lock-keyword-face)
 			;; XEmacs doesn't have font-lock-builtin-face
 			(if (string-match "XEmacs\\|Lucid" emacs-version)
@@ -633,12 +664,45 @@ can be changed by some entry functions to provide more hilighting.")
 
 
 
+;;; Compatibility functions
+
+(if (not (fboundp 'comint-line-beginning-position))
+    ;; comint-line-beginning-position is defined in Emacs 21
+    (defun comint-line-beginning-position ()
+      "Returns the buffer position of the beginning of the line, after any prompt.
+The prompt is assumed to be any text at the beginning of the line matching
+the regular expression `comint-prompt-regexp', a buffer local variable."
+      (save-excursion (comint-bol nil) (point))))
+
+
+
 ;;; Small functions
 
+(defun sql-magic-go (arg)
+  "Insert \"o\" and call `comint-send-input'.
+`sql-electric-stuff' must be the symbol `go'."
+  (interactive "P")
+  (self-insert-command (prefix-numeric-value arg))
+  (if (and (equal sql-electric-stuff 'go)
+	   (save-excursion
+	     (comint-bol nil)
+	     (looking-at "go\\b")))
+      (comint-send-input)))
+
+(defun sql-magic-semicolon (arg)
+  "Insert semicolon and call `comint-send-input'.
+`sql-electric-stuff' must be the symbol `semicolon'."
+  (interactive "P")
+  (self-insert-command (prefix-numeric-value arg))
+  (if (equal sql-electric-stuff 'semicolon)
+       (comint-send-input)))
+
 (defun sql-accumulate-and-indent ()
   "Continue SQL statement on the next line."
   (interactive)
-  (if (fboundp 'comint-accumulate) (comint-accumulate))
+  (if (fboundp 'comint-accumulate) 
+      (comint-accumulate)
+    (newline))
   (indent-according-to-mode))
 
 ;;;###autoload
@@ -669,11 +733,6 @@ buffer.  The output generated is appended to the buffer and a new prompt
 is generated.  See the In/Out menu in the SQLi buffer for some functions
 that help you navigate through the buffer, the input history, etc.
 
-Put a line with a call to autoload into your `~/.emacs' file for each
-entry function you want to use regularly:
-
-\(autoload 'sql-postgres \"sql\" \"Interactive SQL mode.\" t)
-
 If you have a really complex SQL statement or if you are writing a
 procedure, you can do this in a separate buffer.  Put the new buffer in
 `sql-mode' by calling \\[sql-mode].  The name of this buffer can be
@@ -822,13 +881,13 @@ variable `sql-buffer'.  See `sql-help' on how to create such a buffer."
 This is used to set `sql-alternate-buffer-name' within
 `sql-interactive-mode'."
   (concat (if (string= "" sql-user)
-	      (if (string= "" user-login-name)
+	      (if (string= "" (user-login-name))
 		  ()
-		(concat user-login-name "/"))
+		(concat (user-login-name) "/"))
 	    (concat sql-user "/"))
 	  (if (string= "" sql-database)
 	      (if (string= "" sql-server)
-		  system-name
+		  (system-name)
 		sql-server)
 	    sql-database)))
 
@@ -847,21 +906,21 @@ Inserts SELECT or commas if appropriate."
 		  (progn (forward-char 1) (backward-sexp 1) (point))
 		  (progn (forward-sexp 1) (point))))
       (goto-char (point-max))
-      (cond
-       ;; if empty command line, insert SELECT
-       ((save-excursion (beginning-of-line)
-			(looking-at (concat comint-prompt-regexp "$")))
-	(insert "SELECT "))
-       ;; else if appending to INTO .* (, SELECT or ORDER BY, insert a comma
-       ((save-excursion
-	  (re-search-backward "\\b\\(\\(into\\s-+\\S-+\\s-+(\\)\\|select\\|order by\\) .+"
-			      (save-excursion (beginning-of-line) (point)) t))
-	(insert ", "))
-       ;; else insert a space
-       (t
-	(if (eq (preceding-char) ? )
-	    nil
-	  (insert " "))))
+      (let ((bol (comint-line-beginning-position)))
+	(cond
+	 ;; if empty command line, insert SELECT
+	 ((= bol (point))
+	  (insert "SELECT "))
+	 ;; else if appending to INTO .* (, SELECT or ORDER BY, insert a comma
+	 ((save-excursion
+	    (re-search-backward "\\b\\(\\(into\\s-+\\S-+\\s-+(\\)\\|select\\|order by\\) .+"
+				bol t))
+	  (insert ", "))
+	 ;; else insert a space
+	 (t
+	  (if (eq (preceding-char) ? )
+	      nil
+	    (insert " ")))))
       ;; in any case, insert the column
       (insert column)
       (message "%s" column))))
@@ -877,7 +936,7 @@ Inserts SELECT or commas if appropriate."
 (defun sql-query-placeholders-and-send (proc string)
   "Send to PROC input STRING, maybe replacing placeholders.
 Placeholders are words starting with and ampersand like &this.
-This function is used for `comint-input-sender' if running SQL*Plus on NT."
+This function is used for `comint-input-sender' if using `sql-oracle' on NT."
   (while (string-match "&\\(\\sw+\\)" string)
     (setq string (replace-match 
 		  (read-from-minibuffer
@@ -962,6 +1021,8 @@ For information on how to create multiple SQLi buffers, see
   (setq major-mode 'sql-mode)
   (setq mode-name "SQL")
   (use-local-map sql-mode-map)
+  (if sql-mode-menu
+      (easy-menu-add sql-mode-menu)); XEmacs
   (set-syntax-table sql-mode-syntax-table)
   (make-local-variable 'font-lock-defaults)
   ;; Note that making KEYWORDS-ONLY nil will cause havoc if you try
@@ -1047,8 +1108,7 @@ interpreter output, the hooks on `comint-output-filter-functions' are
 run.
 
 Variable `sql-input-ring-file-name' controls the initialisation of the
-input ring history.  `comint-input-ring-file-name' is temporarily bound
-to `sql-input-ring-file-name' when reading the input history.
+input ring history.
 
 Variables `comint-output-filter-functions', a hook, and
 `comint-scroll-to-bottom-on-input' and
@@ -1076,10 +1136,10 @@ you entered, right above the output it created.
   (setq major-mode 'sql-interactive-mode)
   (setq mode-name "SQLi")
   (use-local-map sql-interactive-mode-map)
+  (if sql-interactive-mode-menu
+      (easy-menu-add sql-interactive-mode-menu)); XEmacs
   (set-syntax-table sql-mode-syntax-table)
   (make-local-variable 'font-lock-defaults)
-  (make-local-variable 'sql-server-type)
-  (setq sql-server-type (default-value 'sql-server-type))
   ;; Note that making KEYWORDS-ONLY nil will cause havoc if you try
   ;; SELECT 'x' FROM DUAL with SQL*Plus, because the title of the column
   ;; will have just one quote.  Therefore syntactic hilighting is
@@ -1092,15 +1152,15 @@ you entered, right above the output it created.
   (setq comment-start "--")
   ;; Abbreviation table init and case-insensitive.  It is not activatet
   ;; by default.
-  (setq local-abbrev-table sql-mode-abbrev-table
-	abbrev-all-caps 1)
+  (setq local-abbrev-table sql-mode-abbrev-table)
+  (setq abbrev-all-caps 1)
   ;; Exiting the process will call sql-stop.
   (set-process-sentinel (get-buffer-process sql-buffer) 'sql-stop)
-  ;; Make input-ring stuff buffer local so that people who want a
-  ;; different history file for each buffer/process/client/whatever can
-  ;; change separator and file-name on the sql-interactive-mode-hook.
-  (make-local-variable 'sql-input-ring-separator)
-  (make-local-variable 'sql-input-ring-file-name)
+  ;; People wanting a different history file for each
+  ;; buffer/process/client/whatever can change separator and file-name
+  ;; on the sql-interactive-mode-hook.
+  (setq comint-input-ring-separator sql-input-ring-separator
+	comint-input-ring-file-name sql-input-ring-file-name)
   ;; Create a usefull name for renaming this buffer later.
   (make-local-variable 'sql-alternate-buffer-name)
   (setq sql-alternate-buffer-name (sql-make-alternate-buffer-name))
@@ -1108,10 +1168,7 @@ you entered, right above the output it created.
   (run-hooks 'sql-interactive-mode-hook)
   ;; Calling the hook before calling comint-read-input-ring allows users
   ;; to set comint-input-ring-file-name in sql-interactive-mode-hook.
-  ;; While reading the history, file-name and history are rebound...
-  (let ((comint-input-ring-file-name sql-input-ring-file-name)
-	(comint-input-ring-separator sql-input-ring-separator))
-    (comint-read-input-ring t)))
+  (comint-read-input-ring t))
 
 (defun sql-stop (process event)
   "Called when the SQL process is stopped.
@@ -1123,11 +1180,7 @@ Writes the input history to a history file using
 
 This function is a sentinel watching the SQL interpreter process.
 Sentinels will always get the two parameters PROCESS and EVENT."
-  ;; Write history.
-  ;; While reading the history, file-name and history are rebound...
-  (let ((comint-input-ring-file-name sql-input-ring-file-name)
-	(comint-input-ring-separator sql-input-ring-separator))
-    (comint-write-input-ring))
+  (comint-write-input-ring)
   (if (and (eq (current-buffer) sql-buffer)
 	   (not buffer-read-only))
       (insert (format "\nProcess %s %s\n" process event))
@@ -1137,6 +1190,7 @@ Sentinels will always get the two parameters PROCESS and EVENT."
 
 ;;; Entry functions for different SQL interpreters.
 
+;;;###autoload
 (defun sql-oracle ()
   "Run sqlplus by Oracle as an inferior process.
 
@@ -1176,14 +1230,16 @@ The default comes from `process-coding-system-alist' and
 	    (setq parameter sql-user)))
       (if (and parameter (not (string= "" sql-database)))
 	  (setq parameter (concat parameter "@" sql-database)))
-      (setq parameter (append (list parameter) sql-oracle-options))
       (if parameter
-	  (set-buffer (apply 'make-comint "SQL" sql-oracle-program nil parameter))
+	  (setq parameter (nconc (list parameter) sql-oracle-options))
+	(setq parameter sql-oracle-options))
+      (if parameter
+	  (set-buffer (apply 'make-comint "SQL" sql-oracle-program nil 
+			     parameter))
 	(set-buffer (make-comint "SQL" sql-oracle-program nil))))
-    (setq sql-server-type 'oracle
-	  sql-prompt-regexp "^SQL> "
-	  sql-prompt-length 5
-	  sql-buffer (current-buffer))
+    (setq sql-prompt-regexp "^SQL> ")
+    (setq sql-prompt-length 5)
+    (setq sql-buffer (current-buffer))
     ;; set sql-mode-font-lock-keywords to something different before
     ;; calling sql-interactive-mode.
     (setq sql-mode-font-lock-keywords sql-mode-oracle-font-lock-keywords)
@@ -1196,6 +1252,7 @@ The default comes from `process-coding-system-alist' and
 
 
 
+;;;###autoload
 (defun sql-sybase ()
   "Run isql by SyBase as an inferior process.
 
@@ -1236,16 +1293,16 @@ The default comes from `process-coding-system-alist' and
 	  (setq params (append (list "-U" sql-user) params)))
       (set-buffer (apply 'make-comint "SQL" sql-sybase-program
 			 nil params)))
-    (setq sql-server-type 'sybase
-	  sql-prompt-regexp "^SQL> "
-	  sql-prompt-length 5
-	  sql-buffer (current-buffer))
+    (setq sql-prompt-regexp "^SQL> ")
+    (setq sql-prompt-length 5)
+    (setq sql-buffer (current-buffer))
     (sql-interactive-mode)
     (message "Login...done")
     (pop-to-buffer sql-buffer)))
 
 
 
+;;;###autoload
 (defun sql-informix ()
   "Run dbaccess by Informix as an inferior process.
 
@@ -1276,16 +1333,16 @@ The default comes from `process-coding-system-alist' and
     (if (string= "" sql-database)
 	(set-buffer (make-comint "SQL" sql-informix-program nil))
       (set-buffer (make-comint "SQL" sql-informix-program nil sql-database "-")))
-    (setq sql-server-type 'informix
-	  sql-prompt-regexp "^SQL> "
-	  sql-prompt-length 5
-	  sql-buffer (current-buffer))
+    (setq sql-prompt-regexp "^SQL> ")
+    (setq sql-prompt-length 5)
+    (setq sql-buffer (current-buffer))
     (sql-interactive-mode)
     (message "Login...done")
     (pop-to-buffer sql-buffer)))
 
 
 
+;;;###autoload
 (defun sql-mysql ()
   "Run mysql by TcX as an inferior process.
 
@@ -1329,16 +1386,16 @@ The default comes from `process-coding-system-alist' and
 	  (setq params (append (list (concat "--user=" sql-user)) params)))
       (set-buffer (apply 'make-comint "SQL" sql-mysql-program
 			 nil params)))
-    (setq sql-server-type 'mysql
-	  sql-prompt-regexp "^mysql>"
-	  sql-prompt-length 6
-	  sql-buffer (current-buffer))
+    (setq sql-prompt-regexp "^mysql>")
+    (setq sql-prompt-length 6)
+    (setq sql-buffer (current-buffer))
     (sql-interactive-mode)
     (message "Login...done")
     (pop-to-buffer sql-buffer)))
 
 
 
+;;;###autoload
 (defun sql-solid ()
   "Run solsql by Solid as an inferior process.
 
@@ -1377,16 +1434,16 @@ The default comes from `process-coding-system-alist' and
 	  (setq params (append (list sql-server) params)))
       (set-buffer (apply 'make-comint "SQL" sql-solid-program
 			 nil params)))
-    (setq sql-server-type 'solid
-	  sql-prompt-regexp "^"
-	  sql-prompt-length 0
-	  sql-buffer (current-buffer))
+    (setq sql-prompt-regexp "^")
+    (setq sql-prompt-length 0)
+    (setq sql-buffer (current-buffer))
     (sql-interactive-mode)
     (message "Login...done")
     (pop-to-buffer sql-buffer)))
 
 
 
+;;;###autoload
 (defun sql-ingres ()
   "Run sql by Ingres as an inferior process.
 
@@ -1417,16 +1474,16 @@ The default comes from `process-coding-system-alist' and
     (if (string= "" sql-database)
 	(set-buffer (make-comint "SQL" sql-ingres-program nil))
       (set-buffer (make-comint "SQL" sql-ingres-program nil sql-database)))
-    (setq sql-server-type 'ingres
-	  sql-prompt-regexp "^\* "
-	  sql-prompt-length 2
-	  sql-buffer (current-buffer))
+    (setq sql-prompt-regexp "^\* ")
+    (setq sql-prompt-length 2)
+    (setq sql-buffer (current-buffer))
     (sql-interactive-mode)
     (message "Login...done")
     (pop-to-buffer sql-buffer)))
 
 
 
+;;;###autoload
 (defun sql-ms ()
   "Run isql by Microsoft as an inferior process.
 
@@ -1470,10 +1527,9 @@ The default comes from `process-coding-system-alist' and
 	(setq params (append params (list "-P"))))
       (set-buffer (apply 'make-comint "SQL" sql-ms-program
 			 nil params)))
-    (setq sql-server-type 'ms
-	  sql-prompt-regexp "^[0-9]*>"
-	  sql-prompt-length 5
-	  sql-buffer (current-buffer))
+    (setq sql-prompt-regexp "^[0-9]*>")
+    (setq sql-prompt-length 5)
+    (setq sql-buffer (current-buffer))
     (sql-interactive-mode)
     (message "Login...done")
     (pop-to-buffer sql-buffer)))
@@ -1513,22 +1569,24 @@ Try to set `comint-output-filter-functions' like this:
       (pop-to-buffer "*SQL*")
     (sql-get-login 'database 'server)
     (message "Login...")
-    ;; username and password are ignored.
-    (let ((params))
+    ;; username and password are ignored.  Jason Beegan suggest using
+    ;; --pset and pager=off instead of \\o|cat.  The later was the
+    ;; solution by Gregor Zych.  Jason's suggestion is the default value
+    ;; for sql-postgres-options.
+    (let ((params sql-postgres-options))
       (if (not (string= "" sql-database))
 	  (setq params (append (list sql-database) params)))
       (if (not (string= "" sql-server))
 	  (setq params (append (list "-h" sql-server) params)))
       (set-buffer (apply 'make-comint "SQL" sql-postgres-program
 			 nil params)))
-    (setq sql-server-type 'postgres
-	  sql-prompt-regexp "^.*> *"
-	  sql-prompt-length 5)
+    (setq sql-prompt-regexp "^.*> *")
+    (setq sql-prompt-length 5)
     ;; This is a lousy hack to prevent psql from truncating it's output
     ;; and giving stupid warnings. If s.o. knows a way to prevent psql
     ;; from acting this way, then I would be very thankful to
     ;; incorporate this (Gregor Zych <zych@pool.informatik.rwth-aachen.de>)
-    (comint-send-string "*SQL*" "\\o \| cat\n")
+    ;; (comint-send-string "*SQL*" "\\o \| cat\n")
     (setq sql-mode-font-lock-keywords sql-mode-postgres-font-lock-keywords)
     (setq sql-buffer (current-buffer))
     (sql-interactive-mode)
